package net.awired.ajsl.persistence.dao.implementation;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import net.awired.ajsl.persistence.dao.implementation.abstracts.AbstractDAOImpl;
import net.awired.ajsl.persistence.dao.interfaces.abstracts.AbstractDAO;
import net.awired.ajsl.persistence.entity.interfaces.abstracts.IdEntity;
import org.hibernate.LockMode;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Restrictions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.stereotype.Component;

@Component
@Scope("prototype")
public class GenericsDAOImpl<ENTITY extends IdEntity<KEY_TYPE>, KEY_TYPE extends Serializable> extends
        HibernateDaoSupport implements AbstractDAO<ENTITY, KEY_TYPE> {

    private static final Logger LOG = LoggerFactory.getLogger(AbstractDAOImpl.class);

    public GenericsDAOImpl(Class<ENTITY> entityClass, Class<KEY_TYPE> keyTypeClass) {
        this.entityClass = entityClass;
        this.keyTypeClass = keyTypeClass;
    }

    protected Class<ENTITY>   entityClass;
    protected Class<KEY_TYPE> keyTypeClass;

    // Session session =
    // getHibernateTemplate().getSessionFactory().getCurrentSession();
    // Transaction transaction = session.beginTransaction();

    /**
     * TODO: allow injection of new any type?
     */
    @Override
    protected HibernateTemplate createHibernateTemplate(SessionFactory sessionFactory) {
        return new AjslHibernateTemplate(sessionFactory);
    }

    // ///////////////////////////////////////////////////////////////
    // requests
    // ///////////////////////////////////////////////////////////////

    // /////////////////////////////////////////////////////
    // find

    @Override
    public ENTITY findById(KEY_TYPE id) throws DataAccessException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("findByID for : " + id);
        }
        ENTITY instance = getHibernateTemplate().get(entityClass, id);
        if (LOG.isTraceEnabled()) {
            if (instance == null) {
                LOG.trace("get successful, no instance found for id :" + id);
            } else {
                LOG.trace("get successful, instance found :" + instance);
            }
        }
        return instance;
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<ENTITY> findByIdList(Collection<KEY_TYPE> idList) throws DataAccessException {
        DetachedCriteria criteria = DetachedCriteria.forClass(entityClass);
        criteria.add(Restrictions.in("id", idList));
        List<ENTITY> entityList = getHibernateTemplate().findByCriteria(criteria);
        return entityList;
    }

    /**
     * Alias of loadAll().
     * 
     * @return
     * @throws DataAccessException
     */
    @Override
    public List<ENTITY> findAll() throws DataAccessException {
        return loadAll();
    }

    @Override
    public List<ENTITY> loadAll() throws DataAccessException {
        return getHibernateTemplate().loadAll(entityClass);
    }

    // ///////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void persist(ENTITY transientInstance) throws DataAccessException {
        LOG.debug("persisting instance : " + transientInstance);
        try {
            this.getHibernateTemplate().persist(transientInstance);
            LOG.debug("persist successful : " + transientInstance);
        } catch (RuntimeException re) {
            LOG.error("persist failed", re);
            throw re;
        }
    }

    @Override
    public void save(ENTITY transientInstance) throws DataAccessException {
        LOG.debug("save instance : " + transientInstance.getClass());
        try {
            this.getHibernateTemplate().save(transientInstance);
            this.getHibernateTemplate().flush();
            LOG.debug("save successful : " + transientInstance);
        } catch (RuntimeException re) {
            LOG.error("save failed", re);
            throw re;
        }
    }

    @Override
    public void update(ENTITY transientInstance) throws DataAccessException {
        this.logger.debug("update instance : " + transientInstance.getClass());
        try {
            this.getHibernateTemplate().update(transientInstance);
            LOG.debug("update successful : " + transientInstance);
        } catch (RuntimeException re) {
            LOG.error("update failed", re);
            throw re;
        }
    }

    @Override
    public ENTITY merge(ENTITY detachedEntity) {
        this.logger.debug("merge instance : " + detachedEntity.getClass());
        try {
            ENTITY PersistentObject = this.getHibernateTemplate().merge(detachedEntity);
            LOG.debug("merge successful : " + detachedEntity);
            return PersistentObject;
        } catch (RuntimeException re) {
            LOG.error("merge failed", re);
            throw re;
        }
    }

    // //////////////////////////////////////////////////////////////////////
    // delete

    /**
     * Alias of deleteAll().
     */
    @Override
    public void removeAll() throws DataAccessException {
        deleteAll();
    }

    /**
     * TODO NOT A NICE VERSION SHOULD WRITE IT TO USE A SINGLE QUERY.
     */
    @Override
    public void deleteAll() throws DataAccessException {
        getHibernateTemplate().deleteAll(this.findAll());
    }

    @Override
    public void delete(ENTITY entity) throws DataAccessException {
        getHibernateTemplate().delete(entity);
    }

    /**
     * TODO: can we have access to this ?
     * TODO: its actually used by nested set dao
     */
    @Override
    public void delete(ENTITY entity, LockMode mode) throws DataAccessException {
        getHibernateTemplate().delete(entity, mode);
    }

}
